public class Test {
	private static final int LIST = 0, HEAP = 1;
	private static final int PERF_LIST_LENGTH = 1000;
	private static final int PERF_HEAP_LENGTH = 1000;
	private static final int PERF_LIST_ITERATIONS = 1000;
	private static final int PERF_HEAP_ITERATIONS = 1000;
	private static final int FUNC_LIST_LENGTH = 20;
	private static final int FUNC_HEAP_LENGTH = 20;
	private static final int[][] FUNC_ELEMS_UNSORTED =
		{
			{ 17, 58,  1,  4,  8,  0, 23,  5 },
			{ 17, 1,  4,  8,  0, 23,  5 },

			{  1,  2,  3,  5,  5,  6,  7,  8 },
			{  1,  2,  3,  5,  5,  6,  7 },

			{  8,  7,  6,  5,  4,  3,  2,  1 },
			{  7,  6,  5,  4,  3,  2,  1 },

			{ 42 },
			{}
		};
	private static final int[][] FUNC_ELEMS_SORTED =
		{
			{  0,  1,  4,  5,  8, 17, 23, 58 },
			{  0,  1,  4,  5,  8, 17, 23 },

			{  1,  2,  3,  5,  5,  6,  7,  8 },
			{  1,  2,  3,  5,  5,  6,  7 },

			{  1,  2,  3,  4,  5,  6,  7,  8 },
			{  1,  2,  3,  4,  5,  6,  7 },

			{ 42 },
			{}
		};

	private static int sortAlg;
	private static Provider prov;


	/* ------------------------- Auxiliary functions ------------------- */

	private static String algoString() {
		switch(sortAlg) {
			case LIST:
				return "[LIST]/Heap";
			case HEAP:
				return "List/[HEAP]";
		}
		return null;
	}

	private static String toString(ComparableListNode list) {
		StringBuffer sb = new StringBuffer();

		while (list!=null) {
			sb.append(list.toString());
			sb.append(' ');
			list = list.getNext();
		}

		return sb.toString();
	}

	private static String toString(Node[] array) {
		StringBuffer sb = new StringBuffer();

		for (int i=0; i<array.length; ++i) {
			sb.append(array[i].toString());
			sb.append(' ');
		}

		return sb.toString();
	}

	/* ------------------------- Test functionality -------------------- */
	private static boolean comp(ListNode root, int[] arr) {
		int i=0;

		if ((root==null) != (arr==null || arr.length==0)) { return false; }

		while (root!=null && i<arr.length) {
			if (root.getOrder()!=arr[i]) {
				return false;
			}
			root = (ListNode) root.getNext();
			++i;
		}

		return root==null && i==arr.length;
	}

	private static ListNode arrToList(int[] arr) {
		ComparableListNode root=null, cur;

		if (arr.length>=1) {
			root = new ListNode(arr[0]);
			cur = root;
			for (int i=1; i<arr.length; ++i) {
				cur.setNext(new ListNode(arr[i]));
				cur = cur.getNext();
			}
		}

		return (ListNode) root;
	}

	private static void testFunctionalityListSort() {
		ComparableListNode root, cur, next;
		boolean totalOk = true;
		boolean intermediateOk;

		root = prov.getList(FUNC_LIST_LENGTH);
		Out.println("Original \"random list\": " + toString(root));
		root = SortUtil.listSort(root);
		Out.println("Sorted \"random list\":   " + toString(root));
		if (root != null) {
			cur = root;
			next = cur.getNext();
			while (cur!=null && next!=null && cur.compareTo(next)<=0) {
				cur = next;
				next = next.getNext();
			}
			intermediateOk = next==null;
			totalOk &= intermediateOk;
			Out.println("Random order test listSort: " + ((intermediateOk)?"Ok":"FAILED"));
		} else {
			Out.println("Return value of listSort is null, test FAILED!");
			totalOk = false;
		}

		for (int test=0; test<FUNC_ELEMS_UNSORTED.length; ++test) {
			root = arrToList(FUNC_ELEMS_UNSORTED[test]);
			Out.println("Original \"test " + test + " list\": " + toString(root));
			root = SortUtil.listSort(root);
			Out.println("Sorted \"test " + test + " list\":   " + toString(root));
			intermediateOk = comp((ListNode) root, FUNC_ELEMS_SORTED[test]);
			totalOk &= intermediateOk;
			Out.println("Standard order test " + test + " listSort: " +
				((intermediateOk)?"Ok":"FAILED"));
		}
		Out.println("   +++++++++ Total list sort test: " + ((totalOk)?"Ok":"FAILED") + " +++++++++");
	}

	private static boolean comp(Node[] node, int[] arr) {
		if (arr.length != node.length) { return false; }

		for (int i=0; i<arr.length; ++i) {
			if (node[i].getOrder() != arr[i]) {
				return false;
			}
		}

		return true;
	}

	private static void testFunctionalityHeapSort() {
		Node[] arr;
		boolean totalOk = true;
		boolean intermediateOk;

		arr = prov.getArray(FUNC_HEAP_LENGTH);
		Out.println("Original \"random array\": " + toString(arr));
		SortUtil.heapSort(arr);
		Out.println("Sorted \"random array\":   " + toString(arr));
		intermediateOk = true;
		for (int i=1; i<arr.length; ++i) {
			if (arr[i-1].compareTo(arr[i])>0) {
				intermediateOk = false;
				break;
			}
		}
		totalOk &= intermediateOk;
		Out.println("Random order test heapSort: " + ((intermediateOk)?"Ok":"FAILED"));

		for (int test=0; test<FUNC_ELEMS_UNSORTED.length; ++test) {
			arr = new Node[FUNC_ELEMS_UNSORTED[test].length];
			for (int i=0; i<arr.length; ++i) {
				arr[i] = new Node(FUNC_ELEMS_UNSORTED[test][i]);
			}
			Out.println("Original \"test " + test + " array\": " + toString(arr));
			SortUtil.heapSort(arr);
			Out.println("Sorted \"test " + test + " array\":   " + toString(arr));
			intermediateOk = comp(arr, FUNC_ELEMS_SORTED[test]);
			totalOk &= intermediateOk;
			Out.println("Standard order test " + test + " heapSort: " +
				((intermediateOk)?"Ok":"FAILED"));
		}
		Out.println("   +++++++++ Total heap sort test: " + ((totalOk)?"Ok":"FAILED") + " +++++++++");
	}

	private static void testFunctionality() {
		switch (sortAlg) {
			case LIST: testFunctionalityListSort(); break;
			case HEAP: testFunctionalityHeapSort(); break;
			default: Out.println("Unexpected sort algorithm."); break;
		}
	}


	/* ------------------------- Test performance ---------------------- */
	private static void testPerformanceListSort() {
		ComparableListNode root;
		long timeStart, timeEnd;
		long timeOverhead;
		long timeSum;

		timeStart = System.currentTimeMillis();
		for (int i=0; i<PERF_LIST_ITERATIONS; ++i) {
			root = prov.getList(PERF_LIST_LENGTH);
		}
		timeEnd = System.currentTimeMillis();
		timeOverhead = timeEnd - timeStart;

		timeStart = System.currentTimeMillis();
		for (int i=0; i<PERF_LIST_ITERATIONS; ++i) {
			root = prov.getList(PERF_LIST_LENGTH);
			root = SortUtil.listSort(root);
		}
		timeEnd = System.currentTimeMillis();
		timeSum = timeEnd - timeStart;

		Out.println("Sorting of " + PERF_LIST_LENGTH +
			" elements (" + PERF_LIST_ITERATIONS + " times)" +
			" with listSort: " +
			(timeSum - timeOverhead) +
			"ms");
	}

	private static void testPerformanceHeapSort() {
		Comparable[] arr;
		long timeStart, timeEnd;
		long timeOverhead;
		long timeSum;

		timeStart = System.currentTimeMillis();
		for (int i=0; i<PERF_HEAP_ITERATIONS; ++i) {
			arr = prov.getArray(PERF_HEAP_LENGTH);
		}
		timeEnd = System.currentTimeMillis();
		timeOverhead = timeEnd - timeStart;

		timeStart = System.currentTimeMillis();
		for (int i=0; i<PERF_HEAP_ITERATIONS; ++i) {
			arr = prov.getArray(PERF_HEAP_LENGTH);
			SortUtil.heapSort(arr);
		}
		timeEnd = System.currentTimeMillis();
		timeSum = timeEnd - timeStart;

		Out.println("Sorting of " + PERF_HEAP_LENGTH +
			" elements (" + PERF_HEAP_ITERATIONS + " times)" +
			" with heapSort: " +
			(timeSum - timeOverhead) +
			"ms");
	}

	private static void testPerformance() {
		switch (sortAlg) {
			case LIST: testPerformanceListSort(); break;
			case HEAP: testPerformanceHeapSort(); break;
			default: Out.println("Unexpected sort algorithm."); break;
		}
	}


 	/* ------------------------- Main  --------------------------------- */
	public static void main(String[] args) {
		char ch;

		sortAlg = LIST;
		prov = new Provider();
		do {
			Out.print("Quit/" + algoString() + "/Functionality/Performance (q/l/h/f/p): ");
			ch = In.read();
			In.readLine();
			switch (ch) {
				case 'l':
					sortAlg = LIST;
					break;
				case 'h':
					sortAlg = HEAP;
					break;
				case 'f':
					testFunctionality();
					break;
				case 'p':
					testPerformance();
					break;
				case 'q':
					Out.println("Bye!");
					break;
				default:
					Out.println("Illegal selection!");
			}
		} while (ch != 'q');
	}
}
